@@ -1,4 +1,4 @@
-/* 
+/*
  * Samba LDB module univention_samaccountname_ldap_check
  *	LDB Module for checking samaccountname adds against external LDAP
  *
@@ -39,7 +39,7 @@
      ** NOTE! The following LGPL license applies to the ldb
      ** library. This does NOT imply that all of Samba is released
      ** under the LGPL
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
@@ -74,6 +74,20 @@
 // From openldap/servers/slapd/slap.h
 #define SLAP_LDAPDN_MAXLEN 8192
 
+#define AUTOPTR_FUNC_NAME(type) type##AutoPtrFree
+#define DEFINE_AUTOPTR_FUNC(type, func) \
+    static inline void AUTOPTR_FUNC_NAME(type)(type **_ptr) \
+    { \
+        if (*_ptr) \
+            (func)(*_ptr); \
+        *_ptr = NULL; \
+    }
+#define AUTOPTR(type) \
+    __attribute__((cleanup(AUTOPTR_FUNC_NAME(type)))) type *
+DEFINE_AUTOPTR_FUNC(char, free);
+DEFINE_AUTOPTR_FUNC(FILE, fclose);
+
+
 char *sid_to_string(const struct dom_sid *sid)
 {
 	char buf[DOM_SID_STR_BUFLEN];
@@ -102,7 +116,7 @@ char *sid_to_string(const struct dom_sid *sid)
 
 static char* read_pwd_from_file(char *filename)
 {
-	FILE *fp;
+	AUTOPTR(FILE) fp = NULL;
 	char line[1024];
 	int len;
 
@@ -138,7 +152,6 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 	bool is_computer = false;
 	bool is_group = false;
 	bool is_user = false;
-	char *usersid;
 	int i, fd[2], nbytes, ret;
 	char target_dn_str[SLAP_LDAPDN_MAXLEN+1] = "";	// initialize with NULs
 
@@ -157,10 +170,9 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 	struct auth_session_info *session_info = (struct auth_session_info *)ldb_get_opaque(ldb, "sessionInfo");
 	struct security_token *sec_token = (struct security_token *)session_info->security_token;
 	struct dom_sid *d_sid = (struct dom_sid *)sec_token->sids;
-	usersid = sid_to_string(d_sid);
+	AUTOPTR(char) usersid = sid_to_string(d_sid);
 	ldb_debug(ldb, LDB_DEBUG_TRACE, ("%s: sid: %s\n"), ldb_module_get_name(module), usersid);
 
-	
 	attribute = ldb_msg_find_element(req->op.add.message, "objectClass");
 	for (i=0; i<attribute->num_values; i++) {
 		if ( !(strcasecmp((const char *)attribute->values[i].data, "computer")) ) {
@@ -173,27 +185,26 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 			is_user = true;
 		}
 	}
-			
+
 	if ( is_computer ) {
 		attribute = ldb_msg_find_element(req->op.add.message, "sAMAccountName");
 		if( attribute == NULL ) {
 			// we can't handle this
 			return ldb_next_request(module, req);
 		}
-			
-		char *opt_name = malloc(5 + attribute->values[0].length + 1);
+
+		AUTOPTR(char) opt_name = malloc(5 + attribute->values[0].length + 1);
 		if (opt_name == NULL) {
 			return ldb_module_oom(module);
 		}
 		sprintf(opt_name, "name=%s", attribute->values[0].data);
 		opt_name[5 + attribute->values[0].length] = 0;
 
-		char *opt_unicodePwd = NULL;
+		AUTOPTR(char) opt_unicodePwd = NULL;
 		attribute = ldb_msg_find_element(req->op.add.message, "unicodePwd");
 		if( attribute != NULL ) {
-			char *unicodePwd_base64;
 			size_t unicodePwd_base64_strlen = BASE64_ENCODE_LEN(attribute->values[0].length);
-			unicodePwd_base64 = malloc(unicodePwd_base64_strlen + 1);
+			AUTOPTR(char) unicodePwd_base64 = malloc(unicodePwd_base64_strlen + 1);
 			if (unicodePwd_base64 == NULL) {
 				return ldb_module_oom(module);
 			}
@@ -204,48 +215,51 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 			}
 			sprintf(opt_unicodePwd, "password=%s", unicodePwd_base64);
 			opt_unicodePwd[9 + unicodePwd_base64_strlen] = 0;
-			free(unicodePwd_base64);
 		} else {
 			ldb_debug(ldb, LDB_DEBUG_WARNING, ("%s: new computer object without initial unicodePwd\n"), ldb_module_get_name(module));
 		}
 
-		char *ldap_master = univention_config_get_string("ldap/master");
-		char *machine_pass = read_pwd_from_file("/etc/machine.secret");
+		AUTOPTR(char) ldap_master = univention_config_get_string("ldap/master");
+		AUTOPTR(char) machine_pass = read_pwd_from_file("/etc/machine.secret");
 		if (machine_pass == NULL) {
 			ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: Error reading /etc/machine.secret\n"), ldb_module_get_name(module));
 			return LDB_ERR_OPERATIONS_ERROR;
 		}
 
-		char *my_hostname = univention_config_get_string("hostname");
-		char *opt_my_samaccoutname = malloc(strlen(my_hostname) + 2);
+		AUTOPTR(char) my_hostname = univention_config_get_string("hostname");
+		AUTOPTR(char) opt_my_samaccoutname = malloc(strlen(my_hostname) + 2);
 		if (opt_my_samaccoutname == NULL) {
 			return ldb_module_oom(module);
 		}
 		sprintf(opt_my_samaccoutname, "%s$", my_hostname);
 		opt_my_samaccoutname[strlen(my_hostname)+1] = 0;
-		char *opt_usersid = malloc(strlen(usersid) + strlen("usersid=") + 1);
+		AUTOPTR(char) opt_usersid = malloc(strlen(usersid) + strlen("usersid=") + 1);
 		if (opt_usersid == NULL) {
 			return ldb_module_oom(module);
 		}
 		sprintf(opt_usersid, "usersid=%s", usersid);
-		free(my_hostname);
 
 		int errno_wait = 0;
 		sighandler_t sh;
 		sh = signal(SIGCHLD, SIG_DFL);
 
-		pipe(fd);
+		if (pipe(fd)) {
+			ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: pipe failed\n"), ldb_module_get_name(module));
+			return LDB_ERR_UNWILLING_TO_PERFORM;
+		}
 
 		int status;
 		int pid=fork();
 		if ( pid < 0 ) {
-
 			ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: fork failed\n"), ldb_module_get_name(module));
 			return LDB_ERR_UNWILLING_TO_PERFORM;
 
 		} else if ( pid == 0 ) {
 			close(fd[0]);   // close reading end
-			dup2(fd[1], STDOUT_FILENO);
+			if (fd[1] != STDOUT_FILENO) {
+				dup2(fd[1], STDOUT_FILENO);
+				close(fd[1]);
+			}
 
 			ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: calling ucs-school-create_windows_computer\n"), ldb_module_get_name(module));
 			if (opt_unicodePwd != NULL) {
@@ -253,7 +267,7 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 			} else {
 				status = execl("/usr/sbin/ucs-school-create_windows_computer", "/usr/sbin/ucs-school-create_windows_computer", "-s", ldap_master, "-P", machine_pass, "-U", opt_my_samaccoutname, "selectiveudm/create_windows_computer", "-o", opt_name, "-o", opt_usersid, NULL);
 			}
- 
+
 			if (status == -1) {     // otherwise es wouldn't be here
 				ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: exec of /usr/sbin/ucs-school-create_windows_computer failed: %s\n"), ldb_module_get_name(module), strerror(errno));
 			}
@@ -269,30 +283,25 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 			signal(SIGCHLD, sh);
 		}
 
-		free(ldap_master);
-		free(machine_pass);
-		free(opt_my_samaccoutname);
-		free(opt_name);
-		free(usersid);
-		free(opt_usersid);
-		if (opt_unicodePwd != NULL) {
-			free(opt_unicodePwd);
-		}
-
 		if( ! WIFEXITED(status) ) {
 			ldb_debug(ldb, LDB_DEBUG_ERROR, "%s: Cannot determine return status of ucs-school-create_windows_computer: %s (%d)\n", ldb_module_get_name(module), strerror(errno_wait), errno_wait);
+			close(fd[0]);   // close reading end
 			return LDB_ERR_UNWILLING_TO_PERFORM;
 		} else if( WEXITSTATUS(status) == 2 ) {
 			ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: ldb_add of machine object is disabled\n"), ldb_module_get_name(module));
+			close(fd[0]);   // close reading end
 			return LDB_ERR_UNWILLING_TO_PERFORM;
 		} else if( WEXITSTATUS(status) == 3 ) {
 			ldb_debug(ldb, LDB_DEBUG_TRACE, ("%s: ldb_add of machine object ignored in dummy mode\n"), ldb_module_get_name(module));
+			close(fd[0]);   // close reading end
 			return LDB_SUCCESS;
 		} else if( WEXITSTATUS(status) == 4 ) {
 			ldb_debug(ldb, LDB_DEBUG_ERROR, "%s: LDB_ERR_ENTRY_ALREADY_EXISTS\n", ldb_module_get_name(module));
+			close(fd[0]);   // close reading end
 			return LDB_ERR_ENTRY_ALREADY_EXISTS;
 		} else if( WEXITSTATUS(status) ) {
 			ldb_debug(ldb, LDB_DEBUG_ERROR, ("%s: unknown error code from ucs-school-create_windows_computer: %d\n"), ldb_module_get_name(module), WEXITSTATUS(status));
+			close(fd[0]);   // close reading end
 			return LDB_ERR_UNWILLING_TO_PERFORM;
 		}
 
@@ -308,10 +317,10 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 		}
 
 		// Trim trailing space
-  		char *end_ptr = target_dn_str + nbytes - 1;
-  		while(end_ptr > target_dn_str && isspace((unsigned char)*end_ptr)) end_ptr--;
-  		// Write new null terminator
-  		*(end_ptr+1) = 0;
+		char *end_ptr = target_dn_str + nbytes - 1;
+		while(end_ptr > target_dn_str && isspace((unsigned char)*end_ptr)) end_ptr--;
+		// Write new null terminator
+		*(end_ptr+1) = 0;
 
 		// Now modify request DN
 		msg = ldb_msg_copy_shallow(req, req->op.add.message);
@@ -336,7 +345,7 @@ static int univention_samaccountname_ldap_check_add(struct ldb_module *module, s
 		if (ret != LDB_SUCCESS) {
 			return ret;
 		}
-	 
+
 		return ldb_next_request(module, down_req);
 
 	} else if ( is_user || is_group ) {
